{
  "templateName": "{{ template_name }}",
  "templateDescription": "{{ template_description }}",
  "deviceType": "{{ device_type }}",
  "deviceRole": "{{ device_role }}",
  "configType": "template",
  "factoryDefault": false,
  "policyId": "{{ policy_id }}",
  "featureTemplateUidRange": [],
  "connectionPreferenceRequired": true,
  "connectionPreference": true,
  "generalTemplates": [
        {% for template in general_templates %}
			{{ template | tojson }}
		{% if not loop.last %},{% endif %}
		{% endfor %}
  ],
  "securityPolicyId": "{{ security_policy_id }}"
}